        ;
        ; F-bird text game in a bootsector
        ;
        ; by Oscar Toledo G.
        ; http://nanochess.org/
        ;
        ; Creation date: Jun/04/2017. A messy unoptimized thing.
        ; Revision date: Jun/05/2017. Better usage of graphic charset.
        ;                             Removed a non-8088 long jump. Added
        ;                             sound. Solved bug when overwriting
        ;                             previous score.
        ;

%define MDA_SUPPORT

        use16
        cpu 8086

%ifdef FAT_BOOT
        jmp bootbr
        nop
        times 0x3B db 0
bootbr:
%endif
        cld             ; Reset direction flag (so stosw increments registers)
        sti             ; Allow interrupts
        push cs
        pop ss
        mov sp,0xb800   ; Point to video segment
SetSegments:
        mov ds,sp       ; Both the source (common access)
        mov es,sp       ; and target segments
        ;
        ; Game restart
        ;
fb21:
        mov ax,0x0002   ; Set 80x25 text mode
        int 0x10        ; Call BIOS
        mov di,next     ; Init variables in video segment (saves big bytes)
%ifdef MDA_SUPPORT
        mov sp,0xb000
        inc byte [di]
        je SetSegments
%endif
        mov ax,0x60a0
        stosw           ; next, bird
        xor ax,ax
        stosw           ; grav, pipe
        stosb
        
        ; Disable text mode cursor
        ; https://wiki.osdev.org/Text_Mode_Cursor#Disabling_the_Cursor
        mov ah, 0x01
        mov ch, 0x3f
        int 0x10

        mov di,0x004a   ; Game title
        call print_string
        db 'F-BIRD'
        mov bp,80       ; Introduce 80 columns of scenery
fb1:    call scroll_scenery
%define SI(n) [si+n-0x0fa2]
        dec bp
        jnz fb1

fb23:   mov ah,0x01     ; Check if key pressed
        int 0x16
        pushf
        xor ax,ax       ; Wait for a key
        int 0x16
        popf
        jnz fb23        ; Jump if key was accumulated, if not already waited for key ;)

        ;
        ; Main loop
        ;
fb12:   mov ax,[bird]   ; Bird falls... 
        add al,ah       ; ...because of gravity...
        mov [bird],al   ; ...into new position.
        and al,0xf8     ; Row is a 5.3 fraction, nullify fraction
        mov ah,0x14     ; Given integer is x8, multiply by 20 to get 160 per line
        mul ah          ; Row into screen
        add ax,$0020    ; Fixed column
        xchg ax,di      ; Pass to DI (AX cannot be used as pointer)
        mov dx,$0d1f    ; Draw body
        mov ax,bp       ; [frame]
        mov bx,-160
        and al,4        ; Wing movement each 4 frames
        jz fb15
        mov ax,$0d1e    ; Draw upper wing
        xchg ax,[bx+di] ; Get character below
        jmp short fb16
        ;
        ; Stars and game over
        ;
fb19:   mov al,$2a      ; '*' Asterisks to indicate crashing
        stosb
        inc di
        stosb
        mov di,0x07CA   
        call print_string
        db 'BONK!'
        mov bp,-100     ; Wait 100 frames
fb20:   call wait_frame 
        jnz fb20
        jmp fb21        ; Restart

fb4:    mov al,[bird]
        cmp al,0x18     ; Make sure the bird doesn't fly free outside screen
        jb fb18
        sub al,0x10     ; Move bird two rows upward
fb18:   cbw             ; Reset gravity
        mov [bird],ax
        mov al,0xb6     ; Flap sound
        out (0x43),al
        mov al,0x90
        out (0x42),al
        mov al,0x4a
        out (0x42),al
        in al,(0x61)
        or al,0x03      ; Turn on sound
        out (0x61),al
fb26:   jmp fb12

fb16:   mov dl,$14      ; Draw body
fb15:   or al,[di]      ; Add another character below
        mov [di],dx     ; Draw body
        mov dl,$10      ; Draw head
        xchg dx,[di+2]  ; Get character below head
        add al,dl
        cmp al,0x40     ; Collision with scenery?
        jnz fb19
        call wait_frame ; Wait for frame
        mov ax,bp       ; [frame]
        and al,7        ; 8 frames have passed?
        jnz fb17        ; No, jump
        inc byte SI(grav) ; Increase gravity
fb17:
        mov al,$20
        mov [bx+di],al  ; Delete bird from screen
        stosb
        inc di
        stosb
        call scroll_scenery     ; Scroll scenery
        call scroll_scenery     ; Scroll scenery
        mov di,bx
        mov al,0xb0
        scasb           ; Passed a column?
        jz fb27
        inc di
        scasb           ; Passed a column?
        jnz fb24
fb27:   mov di,0x008e   ; Show current score
fb25:   dec di
        dec di
        mov ax,0x0c30   ; Convert remaining 0-9 to ASCII, also put color
        xchg [di], ax
        or al, 0x30
        cmp al, '9'
        je fb25
        inc ax
        stosb
fb24:   mov ah,0x01     ; Any key pressed?
        int 0x16
        jz fb26         ; No, go to main loop
        mov ah,0x00
        int 0x16        ; Get key
        dec ah          ; Escape key?
        jne fb4         ; No, jump
quit:   mov al,3
        int 0x10
        int 0x20        ; Exit to DOS or to oblivion (boot sector)
        int 0x19

        ;
        ; Scroll scenery one column at a time
        ;
scroll_scenery:
        ;
        ; Move whole screen
        ;
        mov si,0x00a2   ; Point to row 1, column 1 in SI
        mov di,0x00a0   ; Point to row 1, column 0 in DI
        mov bx,di
fb2:    mov cx,79       ; 79 columns
        repz            ; Scroll!!!
        movsw
        mov ax,0x0e20   ; Clean last character
        stosw
        lodsw           ; Advance source to keep pair source/target
        cmp si,0x0fa2   ; All scrolled?
        jnz fb2         ; No, jump
        ;
        ; Insert houses
        ;
        mov word SI(0x0f9e),0x02df        ; Terrain
        in al,(0x40)    ; Get "random" number
        test al,0x70
        jz fb5
        mov dx,0x0408   ; House of one floor
        mov di,0x0e5e
        mov [bx+di],dx
        test al,0x20    ; Check "random" number
        jz fb3
        mov [di],dx     ; House of two floors
        sub di,bx
fb3:    mov word [di],0x091e ; Add roof
        ;
        ; Check if it's time to insert a column
        ;
fb5:    dec byte SI(next) ; Decrease time (column really) for next pipe
        mov dh,SI(next)
        mov dl,0xb1     ; Leftmost
        mov cl,3
        cmp dh,cl       ; bl = 3,2,1,0 for the four columns making the pipe
        ja fb6		; No yet, jump
        jne fb8         ; No leftmost, jump
        and al,0x07     ; Between 0 and 7
        add al,0x04     ; Between 4 and 11
        mov SI(tall),al ; This will tell how tall the pipe is
        jmp fb7         ; Leftmost, jump

fb8:    mov dl,0xdb
        or dh,dh        ; Rightmost?
        jnz fb7         ; No, jump
        mov dl,0xb0
        dec word SI(pipe) ; Increase total pipes shown
        or byte SI(pipe+1),0xfe
        mov ax,SI(pipe)
        sar ax,cl
        add al,0x50     ; Decrease distance between pipes
        mov SI(next),al ; Time for next pipe
fb7:    mov di,0x013e   ; Start from top of screen
        xchg ax,dx
        mov ah,0x0a
        push ax
        mov dx,0x009e
        mov cl,SI(tall)
fb9:    stosw
        add di,dx
        loop fb9
        mov al,0xc4
        stosw
        add di,0x009e*6+10
        mov al,0xdf     
        stosw
        pop ax
fb10:   add di,dx
        stosw
        cmp di,0x0ea0
        jb fb10
fb6:    ret

        ;
        ; Wait for a frame
        ;
wait_frame:
fb14:   hlt             ; Wait for clock interrupt
        mov ah,0x00     ; Use base clock tick
        int 0x1a
        cmp dx,[tick]
        jz fb14
        mov [tick],dx
        in al,(0x61)
        and al,0xfc     ; Turn off sound
        out (0x61),al
        inc bp          ; Increase frame count
        ret

        ;
        ; Print a string
        ;
print_string:      
        pop si
        mov ah, 0xf     ; white
        db 0x3C		; cmp al,0xab
pr1:    stosw
        cs lodsb
        cmp al,0xbd
        jne pr1
        dec si
        push si
        ret

        times 510 - ($-$$) db 0 ; pad remaining 510 bytes with zeroes

        db 0x55,0xaa    ; Bootable signature

tick:   equ 0x0fa0 ; word

next:   equ tick+2 ; byte
bird:   equ tick+3 ; byte
grav:   equ bird+1 ; byte
pipe:   equ tick+5 ; word

tall:   equ tick+7 ; byte
